@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019
+;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2022
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -149,6 +149,7 @@
     ;; YEAR() A year in two-digit or four-digit format.
     ;; NOTE: Values allowed in four-digit format: 1901 to 2155.
     ;;       Values allowed in two-digit format: 70 to 69, representing years from 1970 to 2069
+    ((boolean) (->0 name args)) ; BOOLEAN Stores TRUE or FALSE values
     ((year) (->0 name args))
     (else (throw 'artanis-err 500 ->mysql-type "Invalid type name `~a'" name))))
 
@@ -314,6 +315,9 @@
     ((#:column-format) "COLUMN_FORMAT")
     ((#:storage) (->value 'storage x))
     ((#:reference-definition) "reference_definition")
+    ((#:signed) "signed")
+    ((#:unsigned) "unsigned")
+    ((#:zerofill) "zerofill")
     (else
      (cond
       ((keyword? x)
@@ -331,8 +335,27 @@
 (define (->postgresql-opts dbd opts)
   (format #t "PostgreSQL migration hasn't been supported yet!~%"))
 
-(define (->sqlite3-opts dbd opts)
-  (format #t "SQLite3 migration hasn't been supported yet!~%"))
+(define (->sqlite3-opts x opts)
+  (define-syntax-rule (->value o x)
+    (format #f "~:@(~a~) ~s" o (kw-arg-ref opts x)))
+  (case x
+    ((#:not-null) "NOT NULL")
+    ((#:null) "NULL")
+    ((#:default) "DEFAULT")
+    ((#:unique) "UNIQUE")
+    ((#:unique-key) (throw 'artanis-err 500 "Unique Key is not supported in sqlite3 during table creation."))
+    ((#:primary-key) "PRIMARY KEY")
+    ((#:key) (throw 'artanis-err 500 "Key is not supported in sqlite3 during table creation."))
+    ((#:auto-increment) "AUTOINCREMENT")
+    ((#:auto-now-once) "DEFAULT CURRENT_TIMESTAMP")
+    (else
+     (cond
+      ((keyword? x)
+       (if (is-exception-opt? x)
+           ""
+           (throw 'artanis-err 500 ->mysql-opts
+                  (format #f "Invalid opts `~a' for SQLite3 table definition!" x)))
+       "")))))
 
 (define *table-builder-opts-handler*
   `((mysql . ,->mysql-opts)
@@ -431,17 +454,11 @@
         (() (values kvs w))
         (else (throw 'artanis-err 500 make-table-setter "Invalid kargs `~a'" next)))))
   (define (->kv kvp) (srfi-1:unzip2 kvp))
-  (define (guard vl)
-    (map (lambda (s)
-           ;; TODO: drop unsafe substrings
-           s)
-         vl))
   (lambda (tname . kargs)
     (let-values (((kvp wcond) (->kvp kargs)))
       (let ((sql (if (string-null? wcond)
                      (let-values (((k v) (->kv kvp)))
-                       (let ((sv (guard v)))
-                         (->sql insert into tname k values sv)))
+                       (->sql insert into tname k values v))
                      ;; NOTE: If there's no cond string (say, `where' clauses), you MUSTN'T
                      ;;       use UPDATE because it'll effect all the records!!! In such case,
                      ;;       you should use INSERT.
@@ -578,17 +595,13 @@
                    (get-conf '(db dbd))))))
   (define (column-drop tname cname)
     (case (get-conf '(db dbd))
-      ((mysql postgresql) (->sql alter table tname drop column cname))
-      ((sqlite3) (throw 'artanis-err 500 column-drop
-                        "SQLite3 doesn't support table column modification!"))
+      ((mysql postgresql sqlite3) (->sql alter table tname drop column cname))
       (else (throw 'artanis-err 500 column-drop
                    "Unsupported DBD!" (get-conf '(db dbd))))))
   (define (column-rename tname old new . type)
     (case (get-conf '(db dbd))
       ((mysql) (let ((t (car type))) (->sql alter table tname change column old new t)))
-      ((postgresql) (->sql alter table tname rename column old new))
-      ((sqlite3) (throw 'artanis-err 500 column-rename
-                        "SQLite3 doesn't support table column modification!"))
+      ((postgresql sqlite3) (->sql alter table tname rename column old new))
       (else (throw 'artanis-err 500 column-rename
                    "Unsupported DBD `~a'!" (get-conf '(db dbd))))))
   (define (index-rename tname old new)
@@ -622,6 +635,21 @@
                               "Invalid rc or conn `~a'!" rc/conn)))))
       (DB-query conn sql))))
 
+(define (make-table-counter rc/conn)
+  (lambda* (tname #:key (key "*") (column "count") (group-by #f) (condition ""))
+    (let ((sql (format #f "select ~acount(~a) as ~a from ~a~a~a;"
+                       (if group-by (string-append group-by ",") "")
+                       key column tname
+                       condition
+                       (if group-by (string-append " group by " group-by) "")))
+          (conn (cond
+                 ((route-context? rc/conn) (DB-open rc/conn))
+                 ((<connection>? rc/conn) rc/conn)
+                 (else (throw 'artanis-err 500 make-table-counter
+                              "Invalid rc or conn `~a'!" rc/conn)))))
+      (DB-query conn sql)
+      (DB-get-all-rows conn))))
+
 ;; NOTE: the name of columns is charactar-caseless, at least in MySQL/MariaDB.
 (define (map-table-from-DB rc/conn)
   (define conn
@@ -635,6 +663,7 @@
   (define builder (make-table-builder conn))
   (define dropper (make-table-dropper conn))
   (define modifier (make-table-modifier conn))
+  (define counter (make-table-counter conn))
   ;; NOTE:
   ;; It maybe inefficient to fetch table-schema without any cache, because the request session may generate
   ;; table-schema each time. Although we may build a cache or delayed mechanism here, there's one reason
@@ -658,7 +687,13 @@
     (define-syntax-rule (-> c) (map (cut normalize-column <> ci?) c))
     (and (srfi-1:every (lambda (x) (memq x schema)) (-> args)) #t))
   (define (table-exists? tname)
-    (DB-query rc/conn (format #f "show tables like '~a';" tname))
+    (case (get-conf '(db dbd))
+      ((mysql) (DB-query rc/conn (format #f "show tables like '~a';" tname)))
+      ((postgresql) (DB-query rc/conn (format #f "select from information_schema.tables where table_schema='public' and table_name='~a';" tname)))
+      ((sqlite3) (DB-query rc/conn (format #f "select * from sqlite_master where type='table' and name='~a'" tname)))
+      (else (throw 'artanis-err 500 table-exists?
+                   "Unsupported DBD `~a'!" (get-conf '(db dbd))))
+      )
     ;; If there's no result, dbi will return #f as the result.
     (DB-get-top-row rc/conn))
   (lambda (cmd tname . args)
@@ -682,6 +717,7 @@
       ;; schema is always in downcase.
       ((schema) (get-table-schema tname))
       ((mod) (->call modifier))
+      ((count) (->call counter))
       (else (throw 'artanis-err 500 map-table-from-DB
                    "Invalid cmd: `~a'" cmd)))))
 
