@@ -261,7 +261,7 @@
 ;; for #:session
 (define (session-maker mode rule keys)
   (define* (%spawn rc #:key (idname "sid") (proc session-spawn)
-                   (path "/") domain (expires 3600) secure (http-only #t))
+                   (path "/") domain (expires (get-conf '(cookie expires))) secure (http-only #t))
     (call-with-values
         (lambda () (proc rc #:expires expires))
       (lambda (sid session)
@@ -277,10 +277,8 @@
       (`(spawn ,sid) (cut %spawn <> #:idname sid <...>))
       (`(spawn ,sid ,proc) (cut %spawn <> #:idname sid #:proc proc <...>))
       (else (throw 'artanis-err 500 session-maker "Invalid config mode: ~a" mode))))
-  (define (get-sid rc idname)
-    (any (lambda (c) (and=> (cookie-ref c idname) car)) (rc-cookie rc)))
   (define (check-it rc idname)
-    (let* ((sid (get-sid rc idname))
+    (let* ((sid (get-sid-from-client-cookie rc idname))
            (session (session-restore (or sid ""))))
       (case session
         ((expired)
@@ -288,7 +286,8 @@
          #f)
         ((not-found) #f)
         (else
-         (session-from-correct-client? session rc)))))
+         (when (get-conf '(session hijackcheck))
+           (session-from-correct-client? session rc))))))
   (lambda (rc cmd . args)
     (match cmd
       ('check (check-it rc "sid"))
@@ -298,11 +297,13 @@
       (`(check-and-spawn ,sid)
        (or (check-it rc sid) (apply spawn-handler rc args)))
       ('spawn (apply spawn-handler rc args))
+      ('get (session-restore (get-sid-from-client-cookie rc "sid")))
+      (`(get ,sid) (session-restore (get-sid-from-client-cookie rc sid)))
       ('drop
-       (session-destory! (get-sid rc "sid"))
+       (session-destroy! (get-sid-from-client-cookie rc "sid"))
        (:cookies-remove! rc "sid"))
       (`(drop ,sid)
-       (session-destory! (get-sid rc sid))
+       (session-destroy! (get-sid-from-client-cookie rc sid))
        (:cookies-remove! rc sid))
       (else (throw 'artanis-err 500 session-maker "Invalid call cmd: ~a" cmd)))))
 
@@ -490,7 +491,9 @@
       (string=? p stored-pw)))
   (define (init-post rc)
     (and (rc-body rc)
-         (generate-kv-from-post-qstr (rc-body rc))))
+         (generate-kv-from-post-qstr
+          (rc-body rc)
+          #:no-evil? (get-conf '(db encodeparams)))))
   (define (default-hmac pw salt)
     ;; We still have sha384, sha512 as options, but I think sha256 is enough.
     (string->sha-256 (string-append pw salt)))
