@@ -250,7 +250,7 @@
       (-> "table ~a change column ~a ~a ~a" name old new type)))
     ((_ table name rename column old new)
      (with-dbd
-      'postgresql
+      '(postgresql sqlite3)
       (-> "table ~a rename column ~a to ~a" name old new)))
     ((_ table old-name rename new-name)
      (with-dbd
@@ -378,8 +378,8 @@
             (fmt (string-concatenate `(,(get-prefix) "~{" ,kp "~a~^" ,(->or/and kp) "~}"))))
        (format #f fmt (map ->quote vals))))
     (else (throw 'artanis-err 500 gen-cond
-                 (format #f "[SQL]~a: invalid condition pattern" (get-prefix))
-                 conds))))
+                 (format #f "[SQL]~a: invalid condition pattern `~a'"
+                         (get-prefix) conds)))))
 
 ;; Order of Precedence in SQL
 ;; It is important to understand how the database evaluates multiple comparisons in the WHERE clause.
